@@ -13,9 +13,7 @@
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
-# License along with this program; if not, write to the
-# Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
-# Boston, MA 02110-1301, USA.
+# License along with this program; if not, see <http://www.gnu.org/licenses/>.
 """Logic for ensuring important dependencies satisfy minimum requirements.
 
 The checks here are supposed to take a negligible amount of time (< 0.2 seconds)
@@ -27,8 +25,8 @@ import os
 import sys
 from gettext import gettext as _
 
-missing_soft_deps = {}
-videosink_factory = None
+MISSING_SOFT_DEPS = {}
+VIDEOSINK_FACTORY = None
 
 
 def _version_to_string(version):
@@ -39,19 +37,19 @@ def _string_to_list(version):
     return [int(x) for x in version.split(".")]
 
 
-class Dependency(object):
+class Dependency:
     """Represents a module or component requirement.
 
     Args:
         modulename (str): The name identifying the component.
-        version_required_string (Optional[str]): The minimum required version,
+        version_required (Optional[str]): The minimum required version,
             if any, formatted like "X.Y.Z".
         additional_message (Optional[str]): Message displayed to the user to
             further explain the purpose of the missing component.
     """
 
-    def __init__(self, modulename, version_required_string=None, additional_message=None):
-        self.version_required_string = version_required_string
+    def __init__(self, modulename, version_required=None, additional_message=None):
+        self.version_required = version_required
         self.modulename = modulename
         self.satisfied = False
         self.version_installed = None
@@ -67,14 +65,15 @@ class Dependency(object):
 
         if not self.component:
             self.satisfied = False
-        elif self.version_required_string is None:
-            self.satisfied = True
         else:
-            formatted_version = self._format_version(self.component)
-            self.version_installed = _version_to_string(formatted_version)
-
-            if formatted_version >= _string_to_list(self.version_required_string):
+            if self.version_required is None:
                 self.satisfied = True
+            else:
+                formatted_version = self._format_version(self.component)
+                self.version_installed = _version_to_string(formatted_version)
+
+                if formatted_version >= _string_to_list(self.version_required):
+                    self.satisfied = True
 
     def _try_importing_component(self):
         """Performs the check.
@@ -110,7 +109,7 @@ class Dependency(object):
         else:
             # Translators: %s is a Python module name or another os component
             message = _("- %s version %s is installed but Pitivi requires at least version %s") % (
-                self.modulename, self.version_installed, self.version_required_string)
+                self.modulename, self.version_installed, self.version_required)
 
         if self.additional_message is not None:
             message += "\n    -> " + self.additional_message
@@ -119,10 +118,11 @@ class Dependency(object):
 
 
 class GIDependency(Dependency):
+    # pylint: disable=abstract-method
 
-    def __init__(self, modulename, apiversion, version_required_string=None, additional_message=None):
+    def __init__(self, modulename, apiversion, version_required=None, additional_message=None):
         self.__api_version = apiversion
-        Dependency.__init__(self, modulename, version_required_string, additional_message)
+        Dependency.__init__(self, modulename, version_required, additional_message)
 
     def _try_importing_component(self):
         try:
@@ -140,6 +140,7 @@ class GIDependency(Dependency):
 
 
 class ClassicDependency(Dependency):
+    # pylint: disable=abstract-method
 
     def _try_importing_component(self):
         try:
@@ -176,6 +177,23 @@ class GstPluginDependency(Dependency):
     def _format_version(self, plugin):
         return _string_to_list(plugin.get_version())
 
+    def __repr__(self):
+        if self.satisfied:
+            return ""
+
+        if not self.component:
+            # Translators: %s is a Python module name or another os component
+            message = _("- %s GStreamer plug-in not found on the system") % self.modulename
+        else:
+            # Translators: %s is a Python module name or another os component
+            message = _("- %s Gstreamer plug-in version %s is installed but Pitivi requires at least version %s") % (
+                self.modulename, self.version_installed, self.version_required)
+
+        if self.additional_message is not None:
+            message += "\n    -> " + self.additional_message
+
+        return message
+
 
 class GstDependency(GIDependency):
 
@@ -191,8 +209,8 @@ class GtkDependency(GIDependency):
 
 class CairoDependency(ClassicDependency):
 
-    def __init__(self, version_required_string):
-        ClassicDependency.__init__(self, "cairo", version_required_string)
+    def __init__(self, version_required):
+        ClassicDependency.__init__(self, "cairo", version_required)
 
     def _format_version(self, module):
         return _string_to_list(module.cairo_version_string())
@@ -218,37 +236,35 @@ def _using_broadway_display():
 
 def _check_videosink():
     from gi.repository import Gst
-    global videosink_factory
+    global VIDEOSINK_FACTORY
 
     # If using GdkBroadwayDisplay make sure not to try to use gtkglsink
     # as it would segfault right away.
-    if not videosink_factory and \
+    if not VIDEOSINK_FACTORY and \
             not _using_broadway_display() and \
             "gtkglsink" in os.environ.get("PITIVI_UNSTABLE_FEATURES", ""):
         sink = Gst.ElementFactory.make("gtkglsink", None)
         if sink:
             res = sink.set_state(Gst.State.READY)
             if res == Gst.StateChangeReturn.SUCCESS:
-                videosink_factory = sink.get_factory()
+                VIDEOSINK_FACTORY = sink.get_factory()
                 sink.set_state(Gst.State.NULL)
 
-    if not videosink_factory:
-        videosink_factory = Gst.ElementFactory.find("gtksink")
+    if not VIDEOSINK_FACTORY:
+        VIDEOSINK_FACTORY = Gst.ElementFactory.find("gtksink")
 
-    return videosink_factory
+    return VIDEOSINK_FACTORY
 
 
-def _check_vaapi():
-    from gi.repository import Gst
-    if "vaapi" in os.environ.get("PITIVI_UNSTABLE_FEATURES", ""):
-        print("Vaapi decoders enabled.")
+def _check_hardware_decoders():
+    if "hwdecoders" in os.environ.get("PITIVI_UNSTABLE_FEATURES", ""):
+        print("Hardware decoders enabled.")
         return
 
-    for feature in Gst.Registry.get().get_feature_list_by_plugin("vaapi"):
-        if isinstance(feature, Gst.ElementFactory):
-            klass = feature.get_klass()
-            if "Decoder" in klass and "Video" in klass:
-                feature.set_rank(Gst.Rank.MARGINAL)
+    from gi.repository import Gst
+    for e in Gst.ElementFactory.list_get_elements(Gst.ELEMENT_FACTORY_TYPE_DECODER, Gst.Rank.NONE):
+        if "Hardware" in e.get_metadata("klass"):
+            e.set_rank(Gst.Rank.NONE)
 
 
 def _check_gst_python():
@@ -261,8 +277,9 @@ def _check_gst_python():
 
 
 class GICheck(ClassicDependency):
-    def __init__(self, version_required_string):
-        ClassicDependency.__init__(self, "gi", version_required_string)
+
+    def __init__(self, version_required):
+        ClassicDependency.__init__(self, "gi", version_required)
 
     def _format_version(self, module):
         return list(module.version_info)
@@ -286,7 +303,7 @@ def check_requirements():
     for dependency in SOFT_DEPENDENCIES:
         dependency.check()
         if not dependency.satisfied:
-            missing_soft_deps[dependency.modulename] = dependency
+            MISSING_SOFT_DEPS[dependency.modulename] = dependency
             print(_("Missing soft dependency:"))
             print(dependency)
 
@@ -308,7 +325,7 @@ def check_requirements():
                 "Make sure you have a gtksink available."))
         return False
 
-    _check_vaapi()
+    _check_hardware_decoders()
 
     return True
 
@@ -321,23 +338,7 @@ def require_version(modulename, version):
     except ValueError:
         print(_("Could not import '%s'. Make sure you have it available.")
               % modulename)
-        exit(1)
-
-
-def get_square_width(video_info):
-    """Applies the pixel aspect ratio to the width of the video info.
-
-    Args:
-        video_info (GstPbutils.DiscovererVideoInfo): The video info.
-
-    Returns:
-        int: The width calculated exactly as GStreamer does.
-    """
-    width = video_info.get_width()
-    par_num = video_info.get_par_num()
-    par_denom = video_info.get_par_denom()
-    # We observed GStreamer does a simple int(), so we leave it like this.
-    return int(width * par_num / par_denom)
+        sys.exit(1)
 
 
 def initialize_modules():
@@ -351,7 +352,7 @@ def initialize_modules():
     except ImportError:
         print(_("Could not import 'gi'. "
                 "Make sure you have pygobject available."))
-        exit(1)
+        sys.exit(1)
 
     require_version("Gtk", GTK_API_VERSION)
     require_version("Gdk", GTK_API_VERSION)
@@ -371,25 +372,29 @@ def initialize_modules():
     require_version("GstController", GST_API_VERSION)
     require_version("GstTranscoder", GST_API_VERSION)
     from gi.repository import Gst
-    from pitivi.configure import get_audiopresets_dir, get_videopresets_dir
     Gst.init(None)
 
     require_version("GstPbutils", GST_API_VERSION)
     from gi.repository import GstPbutils
 
-    # Monky patch a helper method for retrieving the size of a video
-    # when using square pixels.
-    GstPbutils.DiscovererVideoInfo.get_square_width = get_square_width
-
     if not os.environ.get("GES_DISCOVERY_TIMEOUT"):
         os.environ["GES_DISCOVERY_TIMEOUT"] = "5"
 
     require_version("GES", GST_API_VERSION)
     from gi.repository import GES
     res, sys.argv = GES.init_check(sys.argv)
+    assert res
     # Monkey patch deprecated methods to use the new variant by default
     GES.TrackElement.list_children_properties = GES.TimelineElement.list_children_properties
 
+    from pitivi.utils.misc import video_info_get_natural_height, video_info_get_natural_width, video_info_get_rotation
+
+    # Monkey patch a helper method for retrieving the size of a video
+    # when using square pixels.
+    GstPbutils.DiscovererVideoInfo.get_natural_width = video_info_get_natural_width
+    GstPbutils.DiscovererVideoInfo.get_natural_height = video_info_get_natural_height
+    GstPbutils.DiscovererVideoInfo.get_rotation = video_info_get_rotation
+
     from pitivi.utils import validate
     if validate.init() and "--inspect-action-type" in sys.argv:
         try:
@@ -397,9 +402,9 @@ def initialize_modules():
         except IndexError:
             action_type = []
         if validate.GstValidate.print_action_types(action_type):
-            exit(0)
+            sys.exit(0)
         else:
-            exit(1)
+            sys.exit(1)
 
 
 # Package maintainers, this is where you can see the list of requirements.
@@ -417,31 +422,39 @@ def initialize_modules():
 # a specific version requirement, they have the "None" value.
 
 GST_API_VERSION = "1.0"
-GST_VERSION = "1.14.2"
+GST_VERSION = "1.17.90"
 GTK_API_VERSION = "3.0"
 GLIB_API_VERSION = "2.0"
-HARD_DEPENDENCIES = [GICheck("3.20.0"),
-                     CairoDependency("1.10.0"),
-                     GstDependency("Gst", GST_API_VERSION, GST_VERSION),
-                     GstDependency("GES", GST_API_VERSION, GST_VERSION),
-                     GIDependency("GstTranscoder", GST_API_VERSION),
-                     GIDependency("GstVideo", GST_API_VERSION),
-                     GtkDependency("Gtk", GTK_API_VERSION, "3.20.0"),
+HARD_DEPENDENCIES = [GICheck(version_required="3.20.0"),
+                     CairoDependency(version_required="1.10.0"),
+                     GstDependency("Gst",
+                                   apiversion=GST_API_VERSION,
+                                   version_required=GST_VERSION),
+                     GstDependency("GES",
+                                   apiversion=GST_API_VERSION,
+                                   version_required=GST_VERSION),
+                     GIDependency("GstTranscoder", apiversion=GST_API_VERSION),
+                     GIDependency("GstVideo", apiversion=GST_API_VERSION),
+                     GtkDependency("Gtk",
+                                   apiversion=GTK_API_VERSION,
+                                   version_required="3.20.0"),
                      ClassicDependency("numpy"),
-                     GIDependency("Gio", "2.0"),
+                     GIDependency("Gio", apiversion="2.0"),
                      GstPluginDependency("gtk"),
                      GstPluginDependency("gdkpixbuf"),
                      ClassicDependency("matplotlib"),
+                     GIDependency("Peas", apiversion="1.0"),
+                     GIDependency("PangoCairo", apiversion="1.0"),
                      ]
 
 SOFT_DEPENDENCIES = (
-    GIDependency("GSound", "1.0", None,
-                 _("enables sound notifications when rendering is complete")),
-    GIDependency("Notify", "0.7", None,
-                 _("enables visual notifications when rendering is complete")),
-    GstPluginDependency("libav", None,
-                        _("additional multimedia codecs through the GStreamer Libav library")),
-    GstPluginDependency("debugutilsbad", None,
-                        _("enables a watchdog in the GStreamer pipeline."
-                          " Use to detect errors happening in GStreamer"
-                          " and recover from them")))
+    GIDependency("GSound", apiversion="1.0", version_required=None,
+                 additional_message=_("enables sound notifications when rendering is complete")),
+    GIDependency("Notify", apiversion="0.7", version_required=None,
+                 additional_message=_("enables visual notifications when rendering is complete")),
+    GstPluginDependency("libav", version_required=None,
+                        additional_message=_("additional multimedia codecs through the GStreamer Libav library")),
+    GstPluginDependency("debugutilsbad", version_required=None,
+                        additional_message=_("enables a watchdog in the GStreamer pipeline."
+                                             " Use to detect errors happening in GStreamer"
+                                             " and recover from them")))
